#!/usr/bin/env python

import os
import sys
import errno
import json
import time
import tempfile
import subprocess
from jetpack import util, amqp

IS_WINDOWS = True if os.name == 'nt' else False
if IS_WINDOWS:
    CYCLECLOUD_HOME = os.getenv('CYCLECLOUD_HOME', 'C:\\cycle\\jetpack')
else:
    CYCLECLOUD_HOME = os.getenv('CYCLECLOUD_HOME', '/opt/cycle/jetpack')

ROUTING_KEY = os.getenv('CYCLECLOUD_STATUS_KEY', 'cyclecloud.service.status')
CONFIG_FILE = os.path.join(CYCLECLOUD_HOME, 'config', 'jetpack.ini')
SERVICE_DIR = os.path.join(CYCLECLOUD_HOME, 'config', 'service.d')
LOG_DIR = os.path.join(CYCLECLOUD_HOME, 'logs', 'service.d')

# Messages may be concatenated and batched
# But in that case they go to the general Topic and specific listeners
# have to consume the general topic and filter out all the other messages
BATCH_MESSAGES = False

# Send the batch once it passes this threshhold even if there are more messages
# we don't want to send huge messages via AMQP
MAX_BATCH_BYTES = 128000


def _call(filename, full_path, cmd=None):
    '''Shell out to execute a subprocess.  Returns path to a file containing
    the process stdout.'''

    output_file = os.path.join(LOG_DIR, '.'.join([filename, 'out']))
    error_file = os.path.join(LOG_DIR, '.'.join([filename, 'err']))
    with open(output_file, 'w') as out:
        with open(error_file, 'w') as err:
            if not cmd:
                cmd = [full_path]
            status = subprocess.call(cmd, stdout=out, stderr=err)
            if status != 0:
                raise Exception("Failed to execute %s!  See %s for details." %
                                (full_path, error_file))
    return output_file

def handle_batch_script(filename, full_path):
    if not IS_WINDOWS:
        print >>sys.stderr, "Skipping windows batch file: %s" % full_path
        return

    return _call(filename, full_path, ["cmd", "/c", full_path])

def handle_shell_script(filename, full_path):
    if IS_WINDOWS:
        print >>sys.stderr, "Skipping unix shell script: %s" % full_path
        return

    return _call(filename, full_path, ["/bin/bash", "-x", full_path])


def handle_python_script(filename, full_path):
    return _call(filename, full_path, ["python", full_path])

# pylint: disable=W0613
def handle_static_message_file(filename, full_path):
    '''Simply validate the message file.'''
    try:
        with open(full_path) as f:
            json.load(f)  # Validate that at least the msg is JSON format
        return full_path
    except Exception, _:
        print >> sys.stderr, "Status file: %s is not valid JSON.  Skipping." % full_path
        return None


def send_status_messages(config, routing_key, msg_files, total_size):
    '''Sends a JSON Array of Message Objects'''
    count = len(msg_files)
    print "Sending %s messages of size %s to %s" % (count, total_size, routing_key)
    temp = tempfile.NamedTemporaryFile()
    try:
        temp.write("[\n")
        for i, msg_file in enumerate(msg_files):
            if i > 0:
                temp.write(",\n")
            with open(msg_file, 'r') as mf:
                temp.write(mf.read())
        temp.write("\n]\n")
        temp.flush()
        msg = temp.name
        amqp.send(message=None, file=msg, routing_key=routing_key, config=config)
    finally:
        temp.close()



EXTENSION_HANDLERS = {'.bat': handle_batch_script,
                      '.cmd': handle_batch_script,
                      '.exe': handle_batch_script,
                      '.sh': handle_shell_script,
                      '.py': handle_python_script,
                      '.json': handle_static_message_file}


def run():
    print "%s : Publishing status reports..." % time.ctime()
    config = util.parse_config(CONFIG_FILE)
    routing_key = ROUTING_KEY

    # Verify that working directories exist
    for d in [LOG_DIR, SERVICE_DIR]:
        try:
            if not os.path.exists(d):
                os.makedirs(d)
        except OSError as exception:
            if exception.errno != errno.EEXIST:
                raise

    message_files = []
    total_message_count = 0
    current_message_size = 0

    for f in os.listdir(SERVICE_DIR):
        path = os.path.join(SERVICE_DIR, f)
        parts = os.path.splitext(f)
        ext = parts[1] if len(parts) > 1 else ''

        handler = EXTENSION_HANDLERS.get(ext, None)
        if not handler:
            print >>sys.stderr, "No handler registered for file %s with "\
                                "extension: %s " % (f, ext)
            continue
        msg_file = handler(f, path)
        if msg_file:
            message_files.append(msg_file)
            current_message_size += os.path.getsize(msg_file)
            if not BATCH_MESSAGES:
                # If we're not batching, send the message to a specific topic
                msg_category = f[:-len(ext)] if ext else f
                routing_key = '.'.join([ROUTING_KEY, msg_category])

            if not BATCH_MESSAGES or current_message_size >= MAX_BATCH_BYTES:
                send_status_messages(config, routing_key, message_files, current_message_size)
                message_files = []
                current_message_size = 0
            total_message_count += 1

    # Send the final batch of messages
    if message_files:
        send_status_messages(config, routing_key, message_files, current_message_size)

    print "%s : Finished sending %s status reports." % (time.ctime(), total_message_count)



if __name__ == '__main__':
    if len(sys.argv) > 1:
        if sys.argv[1].lower() in ['false', 'no', 'n']:
            BATCH_MESSAGES = False
        print "Setting batch messages to %s" % BATCH_MESSAGES
    if len(sys.argv) > 2:
        try:
            batch_size = int(sys.argv[2])
            MAX_BATCH_BYTES = batch_size
            print "Setting max batch size to %s bytes" % MAX_BATCH_BYTES
        except ValueError, ex:
            print >>sys.stderr, "Invalid argument %s for batch size - not an "\
                                "integer # of bytes" % sys.argv[2]

    run()


